/////////////////////////////////////////////////////////////////////////////
// Name:        TemplateDlg.cpp
// Purpose:     Select DVD menu template dialog
// Author:      Alex Thuering
// Created:     20.10.2009
// RCS-ID:      $Id: TemplateDlg.cpp,v 1.12 2013/11/29 19:12:05 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "TemplateDlg.h"
#include "wxVillaLib/utils.h"
#include "wxVillaLib/rc/loading.png.h"
#include <wx/file.h>
#include <wx/filename.h>
#include <wx/stdpaths.h>
#include <wx/dir.h>

#define TEMPLATES_DIR wxFindDataDirectory(_T("templates"))

//(*InternalHeaders(TemplateDlg)
#include <wx/font.h>
#include <wx/intl.h>
#include <wx/button.h>
#include <wx/string.h>
//*)

//(*IdInit(TemplateDlg)
const long TemplateDlg::ID_TEXTCTRL1 = wxNewId();
const long TemplateDlg::CATEGORY_LIST_BOX_ID = wxNewId();
const long TemplateDlg::THUMBNAILS_ID = wxNewId();
const long TemplateDlg::ID_STATICLINE1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(TemplateDlg,wxDialog)
	//(*EventTable(TemplateDlg)
	//*)
	EVT_THUMBNAILS_DCLICK(THUMBNAILS_ID, TemplateDlg::OnThumbDoubleClick)
END_EVENT_TABLE()

TemplateDlg::TemplateDlg(wxWindow* parent, AspectRatio aspectRatio) {
	//(*Initialize(TemplateDlg)
	wxFlexGridSizer* mainGridSizer;
	wxBoxSizer* titleSizer;
	wxStaticText* titleLabel;
	wxStdDialogButtonSizer* stdDialogButtonSizer;
	wxStaticText* categoryLabel;
	wxBoxSizer* mainSizer;

	Create(parent, wxID_ANY, _("Select template for DVD menus"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("wxID_ANY"));
	SetClientSize(wxSize(634,427));
	mainSizer = new wxBoxSizer(wxVERTICAL);
	mainGridSizer = new wxFlexGridSizer(0, 2, 0, 0);
	mainGridSizer->AddGrowableCol(1);
	mainGridSizer->AddGrowableRow(1);
	categoryLabel = new wxStaticText(this, wxID_ANY, _("Category:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
	wxFont categoryLabelFont(10,wxSWISS,wxFONTSTYLE_NORMAL,wxBOLD,false,_T("Arial"),wxFONTENCODING_DEFAULT);
	categoryLabel->SetFont(categoryLabelFont);
	mainGridSizer->Add(categoryLabel, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	titleSizer = new wxBoxSizer(wxHORIZONTAL);
	titleSizer->Add(50,-1,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	titleLabel = new wxStaticText(this, wxID_ANY, _("Title:"), wxDefaultPosition, wxDefaultSize, 0, _T("wxID_ANY"));
	wxFont titleLabelFont(10,wxSWISS,wxFONTSTYLE_NORMAL,wxBOLD,false,_T("Arial"),wxFONTENCODING_DEFAULT);
	titleLabel->SetFont(titleLabelFont);
	titleSizer->Add(titleLabel, 0, wxRIGHT|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 4);
	m_titleText = new wxTextCtrl(this, ID_TEXTCTRL1, _("Disc Title"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_TEXTCTRL1"));
	wxFont m_titleTextFont(10,wxSWISS,wxFONTSTYLE_NORMAL,wxNORMAL,false,wxEmptyString,wxFONTENCODING_DEFAULT);
	m_titleText->SetFont(m_titleTextFont);
	titleSizer->Add(m_titleText, 1, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	titleSizer->Add(50,-1,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	mainGridSizer->Add(titleSizer, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	m_categoryListBox = new wxListBox(this, CATEGORY_LIST_BOX_ID, wxDefaultPosition, wxSize(100,-1), 0, 0, wxLB_SINGLE|wxLB_NEEDED_SB, wxDefaultValidator, _T("CATEGORY_LIST_BOX_ID"));
	m_categoryListBox->SetSelection( m_categoryListBox->Append(_("All")) );
	wxFont m_categoryListBoxFont(10,wxSWISS,wxFONTSTYLE_NORMAL,wxNORMAL,false,wxEmptyString,wxFONTENCODING_DEFAULT);
	m_categoryListBox->SetFont(m_categoryListBoxFont);
	mainGridSizer->Add(m_categoryListBox, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	m_thumbnails = new wxThumbnails(this,THUMBNAILS_ID);
	mainGridSizer->Add(m_thumbnails, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 2);
	mainSizer->Add(mainGridSizer, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 4);
	StaticLine1 = new wxStaticLine(this, ID_STATICLINE1, wxDefaultPosition, wxSize(10,-1), wxLI_HORIZONTAL, _T("ID_STATICLINE1"));
	mainSizer->Add(StaticLine1, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
	stdDialogButtonSizer = new wxStdDialogButtonSizer();
	stdDialogButtonSizer->AddButton(new wxButton(this, wxID_OK, wxEmptyString));
	stdDialogButtonSizer->AddButton(new wxButton(this, wxID_CANCEL, _("&No template")));
	stdDialogButtonSizer->Realize();
	mainSizer->Add(stdDialogButtonSizer, 0, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(mainSizer);
	SetSizer(mainSizer);
	Layout();
	Center();

	Connect(CATEGORY_LIST_BOX_ID,wxEVT_COMMAND_LISTBOX_SELECTED,(wxObjectEventFunction)&TemplateDlg::OnCategorySelect);
	//*)
    m_aspectRatio = aspectRatio;
    if (m_aspectRatio == ar16_9)
    	SetSize(wxSize(798, -1));
    m_thumbnails->SetCaption(_("Templates"));
    m_thumbnails->SetThumbImageSize(m_aspectRatio == ar16_9 ? 205 : 145, 54);
    
    // load categories
	wxString dir = wxFindFirstFile(TEMPLATES_DIR + wxT("*"), wxDIR);
	while (!dir.IsEmpty()) {
		wxString subDir = dir.AfterLast(wxFILE_SEP_PATH);
		if (subDir != wxT("CVS") && !subDir.StartsWith(wxT("."))) {
			m_categoryListBox->Append(wxGetTranslation(subDir.c_str()));
			m_dirs.push_back(subDir);
		}
		dir = wxFindNextFile();
	}
	wxString tempateDir = wxStandardPaths::Get().GetUserDataDir() + wxFILE_SEP_PATH + wxT("templates");
	if (wxDir::Exists(tempateDir)) {
		dir = wxFindFirstFile(tempateDir + wxFILE_SEP_PATH + wxT("*"), wxDIR);
		while (!dir.IsEmpty()) {
			wxString subDir = dir.AfterLast(wxFILE_SEP_PATH);
			if (!subDir.StartsWith(wxT(".")) && m_dirs.Index(subDir) == wxNOT_FOUND) {
				m_categoryListBox->Append(wxGetTranslation(subDir.c_str()));
				m_dirs.push_back(subDir);
			}
			dir = wxFindNextFile();
		}
	}
    wxCommandEvent evt;
    OnCategorySelect(evt);
	
	m_titleText->SetFocus();
	stdDialogButtonSizer->GetAffirmativeButton()->SetDefault();
}

TemplateDlg::~TemplateDlg() {
	//(*Destroy(TemplateDlg)
	//*)
}

wxString TemplateDlg::GetTemplate() {
	return m_thumbnails->GetSelectedItem() ? m_thumbnails->GetSelectedItem()->GetFilename() : wxT("");
}

wxString TemplateDlg::GetTitle() {
	return m_titleText->GetValue();
}

void TemplateDlg::OnThumbDoubleClick(wxCommandEvent& event) {
	EndModal(wxID_OK);
}

void TemplateDlg::LoadThumbnails(wxString dir) {
	if (!wxDir::Exists(dir))
		return;
	wxString fname = wxFindFirstFile(dir + wxFILE_SEP_PATH + _T("*.dvdt"));
	while (!fname.IsEmpty()) {
		if ((m_aspectRatio != ar16_9 && !fname.EndsWith(wxT("WS.dvdt")))
				|| (m_aspectRatio == ar16_9 && (fname.EndsWith(wxT("WS.dvdt"))
						|| !wxFile::Exists(fname.substr(0, fname.length() - 5) + wxT("WS.dvdt"))))) {
			wxLogNull log;
			wxImage img;
			wxString imgFile = fname.BeforeLast(wxT('.')) + wxT(".png");
			if (!wxFile::Exists(imgFile)) {
				DVD dvd;
				if (dvd.Open(fname))
					dvd.RenderThumbnail(imgFile);
			}
			img.LoadFile(imgFile);
			wxThumb* thumb = new wxThumb(img, wxT(""), fname);
			m_thumbnails->InsertItem(thumb);
		}
		fname = wxFindNextFile();
	}
}

void TemplateDlg::OnCategorySelect(wxCommandEvent &event) {
	m_thumbnails->Clear();
	wxString tempateDir = wxStandardPaths::Get().GetUserDataDir() + wxFILE_SEP_PATH + wxT("templates");
	if (m_categoryListBox->GetSelection() == 0) {
		for (wxArrayString::const_iterator it = m_dirs.begin(); it != m_dirs.end(); it++) {
			LoadThumbnails(TEMPLATES_DIR + *it);
			LoadThumbnails(tempateDir + wxFILE_SEP_PATH + *it);
		}
	} else {
		wxString subDir = m_dirs[m_categoryListBox->GetSelection() - 1];
		LoadThumbnails(TEMPLATES_DIR + subDir);
		LoadThumbnails(tempateDir + wxFILE_SEP_PATH + subDir);
	}
	m_thumbnails->SortItems();
	
	if (m_thumbnails->GetItemCount() > 0)
		m_thumbnails->SetSelected(0);
	m_thumbnails->Update();
}
